@@ -4,7 +4,7 @@
  *******************************************************************/
 
 #ifndef DROPBEAR_VERSION
-#define DROPBEAR_VERSION "2024.86"
+#define DROPBEAR_VERSION "2025.87"
 #endif
 
 /* IDENT_VERSION_PART is the optional part after "SSH-2.0-dropbear". Refer to RFC4253 for requirements. */
@@ -165,6 +165,18 @@
 #define DROPBEAR_ECC_384 (DROPBEAR_ECC)
 #define DROPBEAR_ECC_521 (DROPBEAR_ECC)
 
+/* Only include necessary ECC curves building libtomcrypt */
+#define LTC_NO_CURVES
+#if DROPBEAR_ECC_256
+#define LTC_ECC256
+#endif
+#if DROPBEAR_ECC_384
+#define LTC_ECC384
+#endif
+#if DROPBEAR_ECC_521
+#define LTC_ECC521
+#endif
+
 #define DROPBEAR_LTC_PRNG (DROPBEAR_ECC)
 
 /* RSA can be vulnerable to timing attacks which use the time required for
@@ -208,6 +220,9 @@
 #define DROPBEAR_SK_ED25519 ((DROPBEAR_SK_KEYS) && (DROPBEAR_ED25519))
 #endif
 
+#define DROPBEAR_PQHYBRID (DROPBEAR_SNTRUP761 || DROPBEAR_MLKEM768)
+#define DROPBEAR_CURVE25519_DEP (DROPBEAR_CURVE25519 || DROPBEAR_PQHYBRID)
+
 /* Dropbear only uses server-sig-algs, only needed if we have rsa-sha256 pubkey auth */
 #define DROPBEAR_EXT_INFO ((DROPBEAR_RSA_SHA256) \
 		&& ((DROPBEAR_CLI_PUBKEY_AUTH) || (DROPBEAR_SVR_PUBKEY_AUTH)))
@@ -241,16 +256,41 @@
 #define MAX_STRING_LEN (MAX(MAX_CMD_LEN, 2400)) /* Sun SSH needs 2400 for algos,
                                                    MAX_CMD_LEN is usually longer */
 
-/* For a 4096 bit DSS key, empirically determined */
-#define MAX_PUBKEY_SIZE 1700
-/* For a 4096 bit DSS key, empirically determined */
+
+/* Key type sizes are ordered large to small, all are
+ determined empirically, and rounded up */
+#if DROPBEAR_RSA
+/* 4096 bit RSA key */
+#define MAX_PUBKEY_SIZE 600
 #define MAX_PRIVKEY_SIZE 1700
+#elif DROPBEAR_DSS
+#define MAX_PUBKEY_SIZE 500
+#define MAX_PRIVKEY_SIZE 500
+#else
+/* 521 bit ecdsa key */
+#define MAX_PUBKEY_SIZE 200
+#define MAX_PRIVKEY_SIZE 200
+#endif
+
+/* For kex hash buffer, worst case size for Q_C || Q_S || K */
+#if DROPBEAR_MLKEM768
+#define MAX_KEX_PARTS (2*4 + 1184 + 1088 + 32*2 + 68)
+#elif DROPBEAR_SNTRUP761
+/* 2337 */
+#define MAX_KEX_PARTS (2*4 + 1158 + 1039 + 32*2 + 68)
+#elif DROPBEAR_DH_GROUP16
+/* 4096 bit group */
+#define MAX_KEX_PARTS (3 * 520)
+#else
+/* Sufficent for 2048 bit group14, or ecdsa521 */
+#define MAX_KEX_PARTS 1000
+#endif
 
 #define MAX_HOSTKEYS 4
 
 /* The maximum size of the bignum portion of the kexhash buffer */
-/* Sect. 8 of the transport rfc 4253, K_S + e + f + K */
-#define KEXHASHBUF_MAX_INTS (1700 + 130 + 130 + 130)
+/* K_S + Q_C + Q_S + K */
+#define KEXHASHBUF_MAX_INTS (MAX_PUBKEY_SIZE + MAX_KEX_PARTS)
 
 #define DROPBEAR_MAX_SOCKS 2 /* IPv4, IPv6 are all we'll get for now. Revisit
 								in a few years time.... */
@@ -410,6 +450,11 @@
 #endif
 #define DROPBEAR_DSS 1
 
+#if defined(DROPBEAR_RSA_SHA1)
+#undef DROPBEAR_RSA_SHA1
+#endif
+#define DROPBEAR_RSA_SHA1 1
+
 #if defined(DROPBEAR_USE_SSH_CONFIG)
 #undef DROPBEAR_USE_SSH_CONFIG
 #endif
